from abc import abstractmethod

from Connection import Connection
import json


class DB:
    _connection = None
    _config = dict()

    def __init__(self, config: dict = {}):
        with open("config/database.json", "r", encoding="utf-8") as fp:
            self._config = json.load(fp)
        if config:
            self._config = dict(self._config, **config)

    def connect(self, config: dict = {}):
        """
        :param config: dict
        :return: None
        """
        if not self._connection:
            config = dict(self._config, **config)
            self._connection = Connection.instance(config)

    def query(self, sql, bind: tuple = ()):
        return self._connection.query(sql, bind)

    def execute(self, sql, bind: tuple = ()):
        self._connection.execute(sql, bind)

    def __getattr__(self, name, *args):
        return getattr(self, "connect")

    def __str__(self):
        return "config:%s" % self._config


if __name__ == '__main__':
    db = DB()
    db.connect({
        "database": "sex"
    })
    db.execute("update book set type='垃圾' where book_id=1")




import pymongo
from py3db.log.Log import Log


class MongoDb(object):
    def __init__(self, client="mongodb://localhost:27017/", db_name="test", collection=None):
        self.client = pymongo.MongoClient(client)
        self.db = self.client[db_name]
        self.log = Log("mongodb.log")
        if collection:
            self.collection = self.db[collection]

    def insert_one(self, data_dict):
        try:
            self.collection.insert_one(data_dict)
        except Exception as e:
            self.log.call_error()
            self.log.error(str(e))

    def inserts(self, data_dict_list):
        if not self.collection:
            print("连接失败，无法插入")
            return
        self.collection.insert_many(data_dict_list)

    def find_one(self, search_dict):
        return self.collection.find_one(search_dict)

    def finds(self, search_dict):
        return self.collection.find(search_dict)

    def delete_one(self, delete_dict):
        try:
            self.collection.delete_one(delete_dict)
        except Exception as e:
            self.log.call_error()
            self.log.error(str(e))

    def deletes(self, delete_dict):
        try:
            self.collection.delete_many(delete_dict)
        except Exception as e:
            self.log.call_error()
            self.log.error(str(e))

    def update_one(self, filter_dict, update_dict):
        try:
            self.collection.update_one(filter_dict, {"$set": update_dict})
        except Exception as e:
            self.log.call_error()
            self.log.error(str(e))

    def updates(self, filter_dict, update_dict):
        try:
            self.collection.update_many(filter_dict, {"$set": update_dict})
        except Exception as e:
            self.log.call_error()
            self.log.error(str(e))

    def __str__(self) -> str:
        return ("client:{0}\n"
                + "db:{1}\n"
                + "collection:{2}\n").format(
            self.client,
            self.db,
            self.collection
        )


if __name__ == "__main__":
    mongodb = MongoDb(collection="video")
    mongodb.insert_one({"name": "hello world"})



# @Time : 2021/4/23 10:04 
# @File : mysql.py.py 
# @Email: 1727949032@qq.com
# @Software: PyCharm
__author__ = "GanJanWen"

# -*- coding:utf-8 -*-

from py3db.Connection import Connection
from pymysql import connect


class Mysql(Connection):

    def __init__(self, config: dict = {}):
        self.config = config
        self.db = connect(
            host=self.config["host"],
            user=self.config['username'],
            password=self.config['password'],
            database=self.config["database"]
        )

    def query(self, sql, bind: tuple = ()) -> tuple:
        if bind:
            sql = sql % bind
        cursor = self.db.cursor()
        cursor.execute(sql)
        results = cursor.fetchall()
        return results

    def execute(self, sql: str, bind: list = []):
        if bind:
            sql = sql % bind
        cursor = self.db.cursor()
        try:
            cursor.execute(sql)
            self.db.commit()
        except Exception as e:
            print(str(e))
            self.db.rollback()


if __name__ == '__main__':
    mysql = Mysql()
    mysql.connect()



import abc
import json
from hashlib import md5
from pickle import dumps


class Connection(metaclass=abc.ABCMeta):
    _instance = {}
    _link_id = None
    config = {}

    def get_config(self, config: str = ""):
        if config == "":
            return self.config
        else:
            return self.config.get(config)

    @classmethod
    def create_instance(cls, module_name: str, class_name: str, config: dict = {}):
        module_meta = __import__(
            module_name, globals(), locals(), [class_name])
        class_meta = getattr(module_meta, class_name)
        print(config)
        obj = class_meta(config)
        return obj

    @classmethod
    def instance(cls, config: dict = {}):
        md = md5()
        md.update(dumps(config))
        name = md.hexdigest()
        if not cls._instance.get(name):
            cls._instance[name] = cls.create_instance(
                'py3db.connector',
                config['type'].capitalize(),
                config
            )
        return cls._instance[name]

    @abc.abstractmethod
    def query(self, sql: str, bind: list = []):
        pass

    @abc.abstractmethod
    def execute(self, sql: str, bind: list = []):
        pass


if __name__ == "__main__":
    Connection.instance({
        "type": "mysql"
    })



from functools import wraps
from os.path import join
from os import getcwd
from datetime import datetime


def singleton(cls):
    _instance = {}

    @wraps(cls)
    def _singleton(*args, **kwargs):
        if cls not in _instance:
            _instance[cls] = cls(*args, **kwargs)
        return _instance[cls]

    return _singleton


class Time:
    def __init__(self, cls):
        wraps(cls)(self)

    def __call__(self, *args, **kwargs):
        self.__func()
        print("时间:", datetime.now())
        return self.__wrapped__(*args, **kwargs)


@singleton
class Log:
    def __init__(self, file_name):
        self.file_name = file_name

    def _write_log(self, level, msg):
        with open(self.file_name, "a", encoding="utf-8") as fp:
            fp.write("{0}  {1}\n".format(level, msg))

    def critical(self, msg):
        self._write_log("CRITICAL:", msg)

    def error(self, msg):
        self._write_log("ERROR:", msg)

    def warn(self, msg):
        self._write_log("WARN:", msg)

    def info(self, msg):
        self._write_log("INFO:", msg)

    def debug(self, msg):
        self._write_log("DEBUG:", msg)

    def __str__(self):
        return "file_name:{}".format(self.file_name)

    def call_error(self):
        print("This is a error,look {}".format(
            join(getcwd(), self.file_name))
        )


if __name__ == '__main__':
    demo = Log()



# @Time : 2021/4/11 0:30 
# @Author : GanJianWen
# @File : __init__.py.py 
# @Email: 1727949032@qq.com
# @Software: PyCharm

from py3db.elasticsearch.elastic_search import ElasticSearch as ElasticSearch



# @Time : 2021/4/11 0:31 
# @Author : GanJianWen
# @File : elastic_search.py 
# @Email: 1727949032@qq.com
# @Software: PyCharm
from elasticsearch5 import Elasticsearch
from py3db.log.Log import Log


class ElasticSearch(object):
    def __init__(self, es_address_list=['127.0.0.1:9200']):
        self.es = Elasticsearch(
            es_address_list,
            # 启动前嗅探es集群服务器
            sniff_on_start=True,
            # es集群服务器结点连接异常时是否刷新es节点信息
            sniff_on_connection_fail=True,
            # 每60秒刷新节点信息
            sniffer_timeout=60
        )
        self.log = Log("elastic_search.log")

    def search(self, index, doc_type, body):
        return self.es.search(
            index=index,
            doc_type=doc_type,
            body=body
        )

    def insert(self, index, doc_type, body, doc_id):
        try:
            self.es.index(index=index, doc_type=doc_type, body=body, id=doc_id)
        except Exception as e:
            self.log.call_error()
            self.log.error(e)


if __name__ == '__main__':
    es = ElasticSearch()



from py3db.log.Log import Log
from pymysql import connect


class MySql:
    def __init__(self, host, port, user_name, password, database, table=None):
        self.host = host
        self.user_name = user_name
        self.port = port
        self.password = password
        self.database = database
        self.db = None
        self.table_name = table
        self.sql_show = False
        self.log = Log("mysql.log")
        self.create_connect()
        self.results = []
        self.op_list = []

    # @classmethod
    # def table(cls, table_name):
    #     cls.table_name = table_name

    def query(self, sql):
        print(sql) if self.sql_show else None
        cursor = self.db.cursor()
        try:
            cursor.execute(sql)
            return cursor.fetchall()
        except Exception as e:
            self.log.call_error()
            self.log.error(str(e))

    def __str__(self) -> str:
        return "\n".join(["%s:%s" % item for item in self.__dict__.items()])

    def create_connect(self) -> None:
        try:
            self.db = connect(
                host=self.host,
                port=self.port,
                user=self.user_name,
                password=self.password,
                db=self.database,
                charset='utf8mb4'
            )
            if not self.db:
                self.log.call_error()
        except Exception as e:
            self.log.call_error()
            self.log.error(str(e))

    def close_connect(self):
        try:
            self.db.close()
        except Exception as e:
            self.log.call_error()
            self.log.error(str(e))

    def execute(self, sql, variable_name=None, variable=None):
        print(sql) if self.sql_show else None
        cursor = self.db.cursor()
        try:
            cursor.execute(sql)
            self.db.commit()
        except Exception as e:
            self.log.call_error()
            self.log.error("{0} {1} {2}".format(
                e, variable_name, variable
            ))

    def insert_one(self, table=None, columns_list=[], columns_dict={}):
        if not columns_list:
            if not len(columns_dict):
                print('无对应数据')
                return
            else:
                columns_list = columns_dict.values()

        self.table_name = table if table else self.table_name
        value_str = "({})".format(
            ",".join(list(map(lambda x: f"'{x}'" if x is not None else "NULL", columns_list))))
        self.column_name_list = columns_dict.keys() if len(columns_dict) else []
        self.column_name_str = ",".join(self.column_name_list)
        self.column_name_str = f"({self.column_name_str})" if len(
            columns_dict) else ""
        self.op_list = [
            'insert',
            'into',
            self.table_name,
            self.column_name_str,
            'values',
            value_str
        ]
        sql = " ".join(self.op_list)
        self.execute(sql)

    def select(self, table=None, field_list=[], condition_dict={}):
        condition_list = []
        equal_str = "="
        condition_str = " and "
        field_sql = ",".join(field_list) if field_list else "*"
        self.table_name = table if table else self.table_name
        for key, item in condition_dict.items():
            item = item if item else "NULL"
            if isinstance(item, dict):
                equal_str = item.get('compare') if item.get(
                    'compare') else equal_str
                item = item.get('value')
            if item != "NULL":
                condition_list.append("`%s` %s '%s'" % (key, equal_str, item))
            else:
                equal_str = "is"
                condition_list.append("`%s` %s %s" % (key, equal_str, item))
            equal_str = "="
        condition_sql = "1=1" if condition_dict == {
        } else condition_str.join(condition_list)
        sql = "select %s from %s where %s" % (
            field_sql, self.table_name, condition_sql)
        return self.query(sql)

    def update(self, table=None, update_dict: dict = {}, condition_dict: dict = {}):
        update_list = []
        condition_list = []
        equal_str = "="
        self.table_name = table if table else self.table_name
        for key, item in update_dict.items():
            item = item if item else "NULL"
            update_list.append("%s='%s'" % (key, item))
        update_sql = ",".join(update_list)

        for key, item in condition_dict.items():
            item = item if item else "NULL"
            if isinstance(item, dict):
                equal_str = item.get('compare') if item.get(
                    'compare') else equal_str
                item = item.get('value')
            condition_list.append("`%s` %s '%s'" % (key, equal_str, item))
        condition_sql = "1=1" if condition_dict == {
        } else ",".join(condition_list)

        sql = "update {} set {} where {}".format(
            self.table_name, update_sql, condition_sql)
        self.execute(sql)

    def delete(self, table=None, condition_dict={}):
        condition_list = []
        equal_str = "="
        self.table_name = table if table else self.table_name
        for key, item in condition_dict.items():
            item = item if item else "NULL"
            if isinstance(item, dict):
                equal_str = item.get('compare') if item.get(
                    'compare') else equal_str
                item = item.get('value')
            condition_list.append("`%s` %s '%s'" % (key, equal_str, item))
        condition_sql = ",".join(condition_list) if len(
            condition_list) else "1 = 1"
        delete_sql = "delete from %s where %s" % (
            self.table_name, condition_sql)
        self.execute(delete_sql)

    def output_sql(self) -> None:
        self.sql_show = True

    def __del__(self):
        if not self.db and not self:
            self.db.close_connect()


if __name__ == "__main__":
    mysql = MySql("10.18.253.184", 9022, "urrfx4l",
                  "cY9UF6kgQ1mXtQnADi0hQ4UU3M2yUmWD", "crawler")

    mysql.output_sql()
    result = mysql.select("crawling_pub", condition_dict={"id": 895})
    print(result)
